/*************************************************************************\
*                                                                         *
*    File        : aml.v                                                  *
*    Description : Address Matching Logic Module                          *
*    Designer    : Mahmud Galela (mgalela@vlsi.itb.ac.id)                 *
*                                                                         *
*    The function of this module is check destination address field.      *
*    Is it the proper unicast, multicast, or broadcast ?                  *
*                                                                         *
*                                                                         *
\*************************************************************************/

`define INIT_COUNT    (4'b0000)
`define END_COUNT     (4'b1100)
`define BROADCAST     (48'hFFFFFFFFFFFF)
`define ZERO_DA       (48'h000000000000)
`define MULTICAST_SIGN (48'h010000000000)
 
module aml (
  clk, 
  reset_n, 
  data_tap, 
  eth_address,
  SFD_detected,
  address_match,
  broadcast,
  multicast
  );
	
  input		clk, 
                SFD_detected,
      		reset_n;
  input [3:0]	data_tap;
  input [47:0]  eth_address;
  output	address_match, 
                multicast,
  		broadcast;

  reg		address_match, 
                multicast,
  		broadcast,
  		start_count;

  reg [1:0]	state_AML;
  reg [3:0]     AML_count;
  reg [3:0]     data_tap_temp;
  reg [47:0] 	DA;
  parameter     AML_IDLE    =  2'b00,
                AML_RUN     =  2'b01,
                AML_STOP    =  2'b10;
    
always @(posedge clk or negedge reset_n)
  if (!reset_n) 
    state_AML <= AML_IDLE;
  else
    case (state_AML)
	AML_IDLE :  
		    if (SFD_detected)
		    begin
		      start_count <= 1'b1;
	              state_AML <= AML_RUN;
	            end
	            else
	              begin
	                start_count <= 1'b0;
	                DA = `ZERO_DA;
	                state_AML <= AML_IDLE;
	              end
        AML_RUN :  
                      if (AML_count ==`END_COUNT)
	                state_AML <= AML_STOP;
	              else
	              begin
	                state_AML <= AML_RUN;
                 	case(AML_count)
                          4'b0000 : DA[3:0]   = data_tap_temp [3:0];
                          4'b0001 : DA[7:4]   = data_tap_temp [3:0];
                          4'b0010 : DA[11:8]  = data_tap_temp [3:0];
                          4'b0011 : DA[15:12] = data_tap_temp [3:0];
                          4'b0100 : DA[19:16] = data_tap_temp [3:0];
                          4'b0101 : DA[23:20] = data_tap_temp [3:0];
                          4'b0110 : DA[27:24] = data_tap_temp [3:0];
                          4'b0111 : DA[31:28] = data_tap_temp [3:0];
                          4'b1000 : DA[35:32] = data_tap_temp [3:0];
                          4'b1001 : DA[39:36] = data_tap_temp [3:0];
                          4'b1010 : DA[43:40] = data_tap_temp [3:0];
                          4'b1011 : DA[47:44] = data_tap_temp [3:0];
                          default : 
                                    DA = `ZERO_DA;
                        endcase
                      end
        AML_STOP :  
                      begin
                        start_count <= 1'b0;
        	        state_AML <= AML_IDLE;
        	      end
    endcase

always @(posedge clk or negedge reset_n)
begin
  if (!reset_n) 
  begin
     address_match = 1'b0;   
     broadcast     = 1'b0;
     multicast     = 1'b0;  
  end
  else
  if(state_AML == AML_IDLE)
  begin
     address_match = 1'b0;   
     broadcast     = 1'b0;
     multicast     = 1'b0;  
  end
  else
     case(DA)
       eth_address : 
               address_match = 1'b1;
       (eth_address | `MULTICAST_SIGN) :
               begin
                 multicast = 1'b1;
                 address_match = 1'b1;
               end
       `BROADCAST : 
             begin
               broadcast = 1'b1;
               address_match = 1'b1;
             end
       default : 
             address_match = 1'b0;
     endcase
   end        

always @(posedge clk or negedge reset_n)
begin
  if (!reset_n) 
    data_tap_temp <= 4'b0000;   
  else
    data_tap_temp <= data_tap;
end

always @(posedge clk or negedge reset_n)
  if(!reset_n)
    AML_count <= `INIT_COUNT;
  else
    if(!start_count)
      AML_count <= `INIT_COUNT;
  else
      AML_count <= (AML_count + 1'b1);

endmodule 
